@@ -129,6 +129,22 @@ proxy_object_notify_string_cb (GObject*    object,
     g_free (value);
 }
 
+static void
+proxy_widget_boolean_destroy_cb (GtkWidget* proxy,
+                                 GObject*   object)
+{
+    g_signal_handlers_disconnect_by_func (object,
+        proxy_object_notify_boolean_cb, proxy);
+}
+
+static void
+proxy_widget_string_destroy_cb (GtkWidget* proxy,
+                                 GObject*  object)
+{
+    g_signal_handlers_disconnect_by_func (object,
+        proxy_object_notify_string_cb, proxy);
+}
+
 /**
  * katze_property_proxy:
  * @object: a #GObject
@@ -149,10 +165,14 @@ proxy_object_notify_string_cb (GObject*    object,
  *     "uri": the widget created will be particularly suitable for
  *         choosing an existing filename, encoded as an URI.
  *     "font": the widget created will be particularly suitable for
- *         choosing a font from installed fonts.
+ *         choosing a variable-width font from installed fonts.
  *     Since 0.1.6 the following hints are also supported:
  *     "toggle": the widget created will be an empty toggle button. This
  *         is only supported with boolean properties.
+ *         Since 0.1.8 "toggle" creates GtkCheckButton widgets without checkmarks.
+ *     Since 0.2.0 the following hints are also supported:
+ *     "font-monospace": the widget created will be particularly suitable for
+ *         choosing a fixed-width font from installed fonts.
  *
  * Any other values for @hint are silently ignored.
  *
@@ -196,16 +216,19 @@ katze_property_proxy (gpointer     object,
         gchar* notify_property;
         gboolean toggled = katze_object_get_boolean (object, property);
 
+        widget = gtk_check_button_new ();
         if (_hint == g_intern_string ("toggle"))
-            widget = gtk_toggle_button_new ();
+            gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (widget), FALSE);
         else
-            widget = gtk_check_button_new_with_label (gettext (nick));
+            gtk_button_set_label (GTK_BUTTON (widget), gettext (nick));
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), toggled);
         g_signal_connect (widget, "toggled",
                           G_CALLBACK (proxy_toggle_button_toggled_cb), object);
         notify_property = g_strdup_printf ("notify::%s", property);
         g_signal_connect (object, notify_property,
             G_CALLBACK (proxy_object_notify_boolean_cb), widget);
+        g_signal_connect (widget, "destroy",
+            G_CALLBACK (proxy_widget_boolean_destroy_cb), object);
         g_free (notify_property);
     }
     else if (type == G_TYPE_PARAM_STRING && _hint == g_intern_string ("file"))
@@ -256,28 +279,38 @@ katze_property_proxy (gpointer     object,
                               G_CALLBACK (proxy_uri_file_set_cb), object);
         #endif
     }
-    else if (type == G_TYPE_PARAM_STRING && _hint == g_intern_string ("font"))
+    else if (type == G_TYPE_PARAM_STRING && (_hint == g_intern_string ("font")
+        || _hint == g_intern_string ("font-monospace")))
     {
-        int n_families, i;
+        GtkComboBox* combo;
+        gint n_families, i;
         PangoContext* context;
         PangoFontFamily** families;
+        gboolean monospace = _hint == g_intern_string ("font-monospace");
         string = katze_object_get_string (object, property);
 
         widget = gtk_combo_box_new_text ();
+        combo = GTK_COMBO_BOX (widget);
         context = gtk_widget_get_pango_context (widget);
         pango_context_list_families (context, &families, &n_families);
         if (!string)
             string = g_strdup (G_PARAM_SPEC_STRING (pspec)->default_value);
-        for (i = 0; i < n_families; i++)
+        if (string)
         {
-            const gchar* font = pango_font_family_get_name (families[i]);
-            gtk_combo_box_append_text (GTK_COMBO_BOX (widget), font);
-            if (string && !strcmp (font, string))
-                gtk_combo_box_set_active (GTK_COMBO_BOX (widget), i);
+            gint j = 0;
+            for (i = 0; i < n_families; i++)
+            {
+                const gchar* font = pango_font_family_get_name (families[i]);
+                if (monospace != pango_font_family_is_monospace (families[i]))
+                    continue;
+                gtk_combo_box_append_text (combo, font);
+                if (!g_ascii_strcasecmp (font, string))
+                    gtk_combo_box_set_active (combo, j);
+                j++;
+            }
         }
         gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (
-            gtk_combo_box_get_model (GTK_COMBO_BOX (widget))),
-                                     0, GTK_SORT_ASCENDING);
+            gtk_combo_box_get_model (combo)), 0, GTK_SORT_ASCENDING);
         g_signal_connect (widget, "changed",
                           G_CALLBACK (proxy_combo_box_text_changed_cb), object);
         g_free (families);
@@ -298,6 +331,8 @@ katze_property_proxy (gpointer     object,
         notify_property = g_strdup_printf ("notify::%s", property);
         g_signal_connect (object, notify_property,
             G_CALLBACK (proxy_object_notify_string_cb), widget);
+        g_signal_connect (widget, "destroy",
+            G_CALLBACK (proxy_widget_string_destroy_cb), object);
         g_free (notify_property);
     }
     else if (type == G_TYPE_PARAM_FLOAT)
@@ -602,6 +637,64 @@ katze_tree_view_get_selected_iter (GtkTreeView*   treeview,
     return FALSE;
 }
 
+/**
+ * katze_strip_mnemonics:
+ * @original: a string with mnemonics
+ *
+ * Parses the given string for mnemonics in the form
+ * "B_utton" or "Button (_U)" and returns a string
+ * without any mnemonics.
+ *
+ * Return value: a newly allocated string without mnemonics
+ *
+ * Since: 0.1.8
+ **/
+gchar*
+katze_strip_mnemonics (const gchar* original)
+{
+  /* A copy of _gtk_toolbar_elide_underscores
+     Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
+     Copied from GTK+ 2.17.1 */
+  gchar *q, *result;
+  const gchar *p, *end;
+  gsize len;
+  gboolean last_underscore;
+
+  if (!original)
+    return NULL;
+
+  len = strlen (original);
+  q = result = g_malloc (len + 1);
+  last_underscore = FALSE;
+
+  end = original + len;
+  for (p = original; p < end; p++)
+    {
+      if (!last_underscore && *p == '_')
+	last_underscore = TRUE;
+      else
+	{
+	  last_underscore = FALSE;
+	  if (original + 2 <= p && p + 1 <= end &&
+              p[-2] == '(' && p[-1] == '_' && p[0] != '_' && p[1] == ')')
+	    {
+	      q--;
+	      *q = '\0';
+	      p++;
+	    }
+	  else
+	    *q++ = *p;
+	}
+    }
+
+  if (last_underscore)
+    *q++ = '_';
+
+  *q = '\0';
+
+  return result;
+}
+
 /**
  * katze_object_has_property:
  * @object: a #GObject
