@@ -133,9 +133,10 @@ typedef enum
 {
     MIDORI_IDENT_MIDORI,
     MIDORI_IDENT_SAFARI,
+    MIDORI_IDENT_IPHONE,
     MIDORI_IDENT_FIREFOX,
     MIDORI_IDENT_EXPLORER,
-    MIDORI_IDENT_CUSTOM
+    MIDORI_IDENT_CUSTOM,
 } MidoriIdentity;
 
 GType
